/* Created 10/13/01; 12:35:57 AM -- Last Updated 12/29/01

12/29/01
* fixed program so it exits on esc or other misc keypresses
* source code now compiles on linux version of tigcc

This is my eql program for the TI 89 and 92+ rewritten in C thanks to what
I'm sure was a monumental effort on the part of the developers of tigcc.
I was originally inspired by the equation library found on the HP 48G
series calculator.  I've written this program for the TI 85, 86, and 89
in TI Basic.  Unfortunately I have since found the 89 version to be buggy
and wrote this program with the intention of replacing the TI Basic version.
This program unlike it's namesake on the HP line does not come with hundreds
of built in equations; rather it is meant to store equations that you use
often to alleviate the need of continual typing and unnecessary reference.

The program is simple to use, just type in an equation you would like to store
into the equation solver (apps then option 9), then start the program, select
a slot to store it in (note if there are no other equations, it will be stored
in the first slot.) and the next time you start the program the equation will
be waiting for you.  Just select the equation you want to put back into the
equation solver and hit use.

A side note, All the equations are stored in a plain text file called eqldat 
that can be edited with the text editor.  As long as you don't screw up the
format of the file that the program expects you can enter and edit equations
by hand using the text editor.  the format of the file is that each equation
is stored on a line by itself and the last line is blank.  It should look
something like this in the text editor

:y=x^2
:x=hideewho+5
:

this stores two equations.  Don't forget to hit enter after the last equation
there must be a blank line at the end.  Also remember this method of data
entry is optional and is honestly simply a bi-product of the way in which
I chose to store the data.

						Hope you enjoy the program,
								Harlock
								
You can send email to:   pfharlock@yahoo.com

If you decide to do something with the source code please send me an email.

*/
#define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#include <tigcclib.h>         // Include All Header Files

int _ti89,_ti92plus;

char *eqldat = NULL;
int eqnum = 0;
int eqsize = 0;

int loadup(void)
{
	FILE *fp;
	int x;
	eqnum = 0;
	FolderAdd($(eqlfold));
	FolderCur($(eqlfold), 1); 
	if ((fp = fopen("main\\eqldat", "a+")))
	{
		rewind(fp);
		for (eqsize = 0; !feof(fp); eqsize++)
		{
			fgetc(fp);
		}
		if(!(eqldat = malloc(eqsize))) {FolderDel($(eqlfold)); return 1;}
		rewind(fp);
		for (x = 0; !feof(fp); x++)
		{
			if ((eqldat[x] = fgetc(fp))=='\r') {eqldat[x] = '\0'; eqnum++;}
		}
		fclose(fp);
	}
	else {FolderDel($(eqlfold)); return 2;}
	return 0;
}

void cleanup(void)
{
	free(eqldat);
	FolderDel($(eqlfold)); 
}

int popup(int numentries, char *title, ...)
{
	va_list args;
	HANDLE handle;
	int x, act;

	va_start(args, title);
TRY	
	handle = PopupNew (title, 0);
	for(x = 0; x<numentries-1; x++)
	{
		PopupAddText (handle, -1, va_arg(args, char*), 0);
	}
	act = PopupDo (handle, CENTER, CENTER, 0);
	HeapFree (handle); 
ONERR
	ERD_process(errCode);
ENDTRY
	return act;	
}

void geteqlist(char **stringlist)
{
	int x;

	for (x = 0; x<eqnum+8; x++) stringlist[x] = NULL;
	
	for (x = 1, stringlist[0] = eqldat; x < eqnum; x++)
	{
		stringlist[x] = strchr(stringlist[x-1], '\0')+1;
	}
	if (eqnum==0) stringlist[0] = NULL;
}

void useeq(char *string)	// arg is stringlist[eq+page-1]
{
TRY
	push_ANSI_string(string);
	push_quantum_pair(EXPR_TAG, EXT_TAG);
	push_parse_text("eqn");
	push_quantum(STORE_TAG);
	NG_rationalESI(top_estack);
ONERR
	ERD_process (errCode);
ENDTRY
}

int addeq(char **stringlist, int eq)	// second arg eq+page
{
	int x;
	HANDLE handle;
	FILE *fp;
	char *temp;
	
	if ((fp = fopen("main\\eqldat", "w")))
	{
		if (eq>eqnum) eq = eqnum+1;
		for (x = 1; x < eq ; x++)
		{
			fprintf(fp, "%s\n", stringlist[x-1]);
		}	
TRY
		push_parse_text("eqn");
		push_quantum_pair(STRING_TAG, EXT_TAG);
		NG_rationalESI(top_estack);
		handle = display_statements (top_estack, 1, 1);
ONERR
		ERD_process(errCode);
ENDTRY
		if (!(temp = malloc((1+strlen(HeapDeref(handle)))*sizeof(char)))) return 1;
		strcpy(temp, ((char *)HeapDeref(handle))+1);
		temp[strlen(temp)-1] = '\0';
		fprintf(fp, "%s\n", temp);
		HeapFree(handle);
		free(temp);
		for (; x <= eqnum; x++)
		{
			fprintf(fp, "%s\n", stringlist[x-1]);
		}
		cleanup();
		if (loadup()) return 5;
		geteqlist(stringlist);
		fclose(fp);
	}
	return 0;
}

void deleq(char **stringlist, int eq)  // second arg eq+page
{
	FILE *fp;
	int x;

	if (eq<=eqnum)
	{
		for (; eq < eqnum; eq++)
		{
			strcpy(stringlist[eq-1], stringlist[eq]);
		}
		eqnum--;
		geteqlist(stringlist);

		if ((fp = fopen("main\\eqldat", "w")))
		{
			for (x = 0; x < eqnum; x++)
			{
				fprintf(fp, "%s\n", stringlist[x]);
			}
			fclose(fp);
		}
	}
}

void _main(void)
{
	int act, eq, page = 0;
	act = popup(4, "Select", "Load Data", "Reset", "Quit");

	if (!act) return;
	
	if (loadup()) return;
	{
		char *stringlist[eqnum+8];
		geteqlist(stringlist);
		switch (act)
		{
			case 2:
			cleanup();
			SymDel($(eqldat)); 
			if (loadup()) return;
			geteqlist(stringlist);
			goto MORE;
			break;
			case 3:
			cleanup();
			return;
			break;
		}
MORE:
		eq = popup(8, "Select", stringlist[page], stringlist[page+1], stringlist[page+2], stringlist[page+3], 
								stringlist[page+4], "More...", "Quit");
		switch (eq)
		{
			case 0:
			case 7:
			cleanup();
			return;
			break;
			case 6:
			if (page+5<=eqnum) page=page+5;
			else page = 0;
			goto MORE;
			break;
			default:
			act = popup(5, "Select", "Use", "Add", "Delete", "Quit");
			switch (act)
			{
				case 1:
				useeq(stringlist[eq+page-1]);
				break;
				case 2:
				addeq(stringlist, eq+page);
				break;
				case 3:
				deleq(stringlist, eq+page);
			}
		}
		cleanup();
	}
}
